package net.sf.sido;

/**
 * An interface that enables path-based access to the properties of an object.
 * 
 * @author Damien Coraboeuf
 * 
 */
public interface PropertyAccessor {

	/**
	 * Separator used for nested properties.
	 */
	String SEPARATOR = ".";

	/**
	 * Gets the value of a property
	 * 
	 * @param propertyPath
	 *            Path to the property
	 * @return Property value or <code>null</code> if not found
	 */
	Object getValue(String propertyPath);

	/**
	 * Sets the value of a property
	 * 
	 * @param propertyPath
	 *            Path to the property
	 * @param value
	 *            Value to set (can be <code>null</code>)
	 * @return This object (allows chaining)
	 */
	PropertyAccessor setValue(String propertyPath, Object value);

}
