package net.sf.sido.factory;

import java.io.IOException;
import java.net.URL;
import java.util.Collection;

import net.sf.sido.DataFactory;
import net.sf.sido.DataObject;
import net.sf.sido.DataSchema;
import net.sf.sido.factory.support.SiDOSchema;

public interface DataFactoryReader {

	/**
	 * Reads a schema from a file.
	 * 
	 * @param url
	 *            URL to the schema file
	 * @param factory
	 *            Factory that is used to create the schema.
	 * @return Schema
	 * @throws IOException
	 *             If the schema file cannot be read
	 */
	DataSchema read(URL url, DataFactory factory) throws IOException;

	/**
	 * Reads schemas from a set of files.
	 *
	 * @param urls
	 *            URLs to the schema files
	 * @param factory
	 *            Factory that is used to create the schema.
	 * @return List of schema - note that the ordering may not be the same that the URLs
	 * @throws IOException
	 *             If the schema files cannot be read
	 */
	Collection<DataSchema> read(Collection<URL> urls, DataFactory factory) throws IOException;

    /**
     * Reads a schema definition (Schema::SSchema) from a URL.
     * @param sidoSchema Schema:: definition
     * @param url URL to read from
     * @return Schema::SSchema
     * @throws IOException If the URL cannot be read
     */
    DataObject readSchema(SiDOSchema sidoSchema, URL url) throws IOException;

}
