package net.sf.sido.support;

import net.sf.sido.DataType;
import net.sf.sido.UndefinedProperty;

import org.apache.commons.lang.Validate;

import com.google.common.base.Predicate;

public abstract class AbstractProperty<T> implements UndefinedProperty<T> {

	private final String name;
	private final boolean nullable;
	private final Class<? super T> type;
	private final boolean extension;
	private final Predicate<T> constraint;

	private DataType holderType;

	public AbstractProperty(String name, boolean nullable,
			Class<? super T> type, boolean extension, Predicate<T> constraint) {
		Validate.notEmpty(name, "Property name is required");
		Validate.notNull(type, "Property type is required");
		this.name = name;
		this.nullable = nullable;
		this.type = type;
		this.extension = extension;
		this.constraint = constraint;
	}

	@Override
	public Predicate<T> getConstraint() {
		return this.constraint;
	}

	@Override
	public DataType getHolderType() {
		return this.holderType;
	}

	@Override
	public String getName() {
		return this.name;
	}

	@Override
	public Class<? super T> getType() {
		return this.type;
	}

	@Override
	public boolean isExtension() {
		return this.extension;
	}

	@Override
	public boolean isNullable() {
		return this.nullable;
	}

	@Override
	public void setHolderType(DataType type) {
		if (this.holderType != null) {
			throw new IllegalStateException(
					"Once defined, the holder type of a property cannot be set any longer.");
		} else {
			this.holderType = type;
		}
	}

	@Override
	public String toString() {
		return name;
	}

}
