package net.sf.sido.support;


import com.google.common.base.Predicate;
import net.sf.sido.DataObject;
import org.apache.commons.lang.ObjectUtils;

import java.util.regex.Pattern;

/**
 * Generator of predicates.
 */
public class DOPredicates {

    public static Predicate<DataObject> equals (final String propertyName, final Object value) {
        return new Predicate<DataObject>() {
            @Override
            public boolean apply(DataObject input) {
                Object candidate = input.get(propertyName);
                return ObjectUtils.equals(value, candidate);
            }
        };
    }

    public static Predicate<DataObject> regex (final String propertyName, String regex) {
        Pattern pattern = Pattern.compile(regex);
        return regex(propertyName, pattern);
    }

    public static Predicate<DataObject> regex (final String propertyName, final Pattern regex) {
        return new Predicate<DataObject>() {
            @Override
            public boolean apply(DataObject input) {
                Object candidate = input.get(propertyName);
                String value = ObjectUtils.toString(candidate, null);
                return regex.matcher(value).matches();
            }
        };
    }

    public static Predicate<DataObject> isNull (final String propertyName) {
        return new Predicate<DataObject>() {
            @Override
            public boolean apply(DataObject input) {
                Object candidate = input.get(propertyName);
                return (candidate == null);
            }
        };
    }

    public static Predicate<DataObject> isNotNull (final String propertyName) {
        return new Predicate<DataObject>() {
            @Override
            public boolean apply(DataObject input) {
                Object candidate = input.get(propertyName);
                return (candidate != null);
            }
        };
    }

}
