package net.sf.sido.support;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

import net.sf.sido.DataFactory;
import net.sf.sido.DataSchema;
import net.sf.sido.DataType;
import net.sf.sido.DataTypeBuilder;

import org.apache.commons.lang.Validate;

import com.google.common.collect.ImmutableList;

public class DefaultDataSchema implements DataSchema {

	private final DataFactory factory;
	private final String name;

	private final Map<String, DataType> types = new LinkedHashMap<String, DataType>();

	public DefaultDataSchema(DataFactory factory, String name) {
		Validate.notNull(factory, "Schema factory is required");
		Validate.notEmpty(name, "Schema name is required");
		this.factory = factory;
		this.name = name;
	}

	@Override
	public void close() {
		for(DataType type : types.values()) {
			type.close();
		}
	}

	@Override
	public DataTypeBuilder createDataTypeBuilder(final String name) {
		return new DefaultDataTypeBuilder(this, name) {
			@Override
			public DataType getType() {
				DataType type = super.getType();
				types.put(name, type);
				return type;
			}
		};
	}

	@Override
	public boolean equals(Object obj) {
		return obj instanceof DataSchema && ((DataSchema) obj).getName().equals(name);
	}

	@Override
	public DataFactory getFactory() {
		return factory;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public DataType getType(String name) {
		return types.get(name);
	}

	@Override
	public Collection<DataType> getTypes() {
		return ImmutableList.copyOf(types.values());
	}

	@Override
	public int hashCode() {
		return name.hashCode();
	}

	@Override
	public String toString() {
		return name;
	}

}
