package net.sf.sido.support;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import net.sf.sido.DOException;
import net.sf.sido.ValidationError;
import net.sf.sido.ValidationResults;

public class DefaultValidationResults implements ValidationResults {

	private final List<ValidationError> errors = new ArrayList<ValidationError>();

	public void addError(String property, DOException error) {
		this.errors.add(new DefaultValidationError(property, error));
	}

	@Override
	public List<ValidationError> getErrors() {
		return this.errors;
	}

	@Override
	public boolean isValid() {
		return this.errors.isEmpty();
	}

	public void merge(String name, ValidationResults childResults) {
		List<ValidationError> list = childResults.getErrors();
		for (ValidationError error : list) {
			error = new DefaultValidationError(name + "." + error.getName(),
					error.getError());
			this.errors.add(error);
		}
	}

    @Override
    public String getDisplayableResults() {
        return getDisplayableResults(Locale.getDefault());
    }

    @Override
    public String getDisplayableResults(Locale locale) {
        return getDisplayableResults(locale, "* {0}\n");
    }

    @Override
    public String getDisplayableResults(Locale locale, String lineFormat) {
        return getDisplayableResults(locale, lineFormat, "{0}");
    }

    @Override
    public String getDisplayableResults(Locale locale, String lineFormat, String resultFormat) {
        MessageFormat mLine = new MessageFormat(lineFormat);
        MessageFormat mResult = new MessageFormat(resultFormat);
        List<String> errors = getDisplayableErrors(locale);
        StringBuilder out = new StringBuilder();
        for (String error : errors) {
            String line = mLine.format(new Object[] {error});
            out.append(line);
        }
        return mResult.format(new Object[] {out});
    }

    @Override
    public List<String> getDisplayableErrors() {
        return getDisplayableErrors(Locale.getDefault());
    }

    @Override
    public List<String> getDisplayableErrors(Locale locale) {
        List<String> messages = new ArrayList<String>(errors.size());
        for (ValidationError error : errors) {
            String message = error.getMessage(locale);
            messages.add(message);
        }
        return messages;
    }
}
